@@ -8,24 +8,19 @@
  *
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation;
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
- *  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
- *  CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
- *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
- *  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
- *  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, 
- *  COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS 
- *  SOFTWARE IS DISCLAIMED.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: hci.h,v 1.76 2005/09/08 11:50:20 holtmann Exp $
  */
 
 #ifndef __HCI_H
@@ -248,6 +243,7 @@ enum {
 #define LMP_EDR_3SLOT	0x80
 
 #define LMP_EDR_5SLOT	0x01
+#define LMP_SNIFF_SUBR	0x02
 #define LMP_AFH_CAP_MST	0x08
 #define LMP_AFH_CLS_MST	0x10
 #define LMP_EDR_ESCO_2M	0x20
@@ -579,6 +575,16 @@ typedef struct {
 
 #define OCF_WRITE_DEFAULT_LINK_POLICY	0x000F
 
+#define OCF_SNIFF_SUBRATE		0x0011
+typedef struct {
+	uint16_t	handle;
+	uint16_t	max_remote_latency;
+	uint16_t	max_local_latency;
+	uint16_t	min_remote_timeout;
+	uint16_t	min_local_timeout;
+} __attribute__ ((packed)) sniff_subrate_cp;
+#define SNIFF_SUBRATE_CP_SIZE 10
+
 /* Host Controller and Baseband */
 #define OGF_HOST_CTL		0x03
 
@@ -1369,6 +1375,17 @@ typedef struct {
 } __attribute__ ((packed)) evt_sync_conn_changed;
 #define EVT_SYNC_CONN_CHANGED_SIZE 9
 
+#define EVT_SNIFF_SUBRATE		0x2E
+typedef struct {
+	uint8_t		status;
+	uint16_t	handle;
+	uint16_t	max_remote_latency;
+	uint16_t	max_local_latency;
+	uint16_t	min_remote_timeout;
+	uint16_t	min_local_timeout;
+} __attribute__ ((packed)) evt_sniff_subrate;
+#define EVT_SNIFF_SUBRATE_SIZE 11
+
 #define EVT_EXTENDED_INQUIRY_RESULT	0x2F
 typedef struct {
 	bdaddr_t	bdaddr;
