@@ -9,24 +9,19 @@
  *
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation;
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
- *  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
- *  CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
- *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
- *  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
- *  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, 
- *  COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS 
- *  SOFTWARE IS DISCLAIMED.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: sdp.h,v 1.13 2005/01/29 03:08:20 holtmann Exp $
  */
 
 #ifndef __SDP_H
@@ -127,6 +122,8 @@ extern "C" {
 #define UDI_TA_SVCLASS_ID		0x112b
 #define AV_SVCLASS_ID			0x112c
 #define SAP_SVCLASS_ID			0x112d
+#define PBAP_PCE_SVCLASS_ID		0x112e
+#define PBAP_PSE_SVCLASS_ID		0x112f
 #define PNP_INFO_SVCLASS_ID		0x1200
 #define GENERIC_NETWORKING_SVCLASS_ID	0x1201
 #define GENERIC_FILETRANS_SVCLASS_ID	0x1202
@@ -139,6 +136,7 @@ extern "C" {
 #define UPNP_L2CAP_SVCLASS_ID		0x1302
 #define VIDEO_SOURCE_SVCLASS_ID		0x1303
 #define VIDEO_SINK_SVCLASS_ID		0x1304
+#define VIDEO_DISTRIBUTION_SVCLASS_ID	0x1305
 
 /*
  * Standard profile descriptor identifiers; note these
@@ -191,6 +189,8 @@ extern "C" {
 #define UDI_TA_PROFILE_ID		UDI_TA_SVCLASS_ID
 #define AV_PROFILE_ID			AV_SVCLASS_ID
 #define SAP_PROFILE_ID			SAP_SVCLASS_ID
+#define PBAP_PCE_PROFILE_ID		PBAP_PCE_PROFILE_ID
+#define PBAP_PSE_PROFILE_ID		PBAP_PSE_PROFILE_ID
 #define PNP_INFO_PROFILE_ID		PNP_INFO_SVCLASS_ID
 #define GENERIC_NETWORKING_PROFILE_ID	GENERIC_NETWORKING_SVCLASS_ID
 #define GENERIC_FILETRANS_PROFILE_ID	GENERIC_FILETRANS_SVCLASS_ID
@@ -203,6 +203,7 @@ extern "C" {
 #define UPNP_L2CAP_PROFILE_ID		UPNP_L2CAP_SVCLASS_ID
 #define VIDEO_SOURCE_PROFILE_ID		VIDEO_SOURCE_SVCLASS_ID
 #define VIDEO_SINK_PROFILE_ID		VIDEO_SINK_SVCLASS_ID
+#define VIDEO_DISTRIBUTION_PROFILE_ID	VIDEO_DISTRIBUTION_SVCLASS_ID
 
 /*
  * Attribute identifier codes
@@ -255,6 +256,7 @@ extern "C" {
 #define SDP_ATTR_SUPPORTED_FEATURES		0x0311
 #define SDP_ATTR_SUPPORTED_FUNCTIONS		0x0312
 #define SDP_ATTR_TOTAL_IMAGING_DATA_CAPACITY	0x0313
+#define SDP_ATTR_SUPPORTED_REPOSITORIES		0x0314
 
 #define SDP_ATTR_SPECIFICATION_ID		0x0200
 #define SDP_ATTR_VENDOR_ID			0x0201
@@ -371,11 +373,13 @@ extern "C" {
  * Values of the flags parameter to sdp_record_register
  */
 #define SDP_RECORD_PERSIST	0x01
+#define SDP_DEVICE_RECORD	0x02
 
 /*
  * Values of the flags parameter to sdp_connect
  */
 #define SDP_RETRY_IF_BUSY	0x01
+#define SDP_WAIT_ON_CLOSE	0x02
 
 /*
  * SDP Error codes
@@ -400,7 +404,7 @@ typedef struct {
  * Should the type of any of these change, you need only make a change here.
  */
 typedef struct {
-	char data[16];
+	uint8_t data[16];
 } uint128_t;
 
 typedef struct {
@@ -453,9 +457,9 @@ typedef struct {
 } sdp_version_t;
 
 typedef struct {
-	char *data;
-	int data_size;
-	int buf_size;
+	uint8_t *data;
+	uint32_t data_size;
+	uint32_t buf_size;
 } sdp_buf_t;
 
 typedef struct {
