@@ -9,24 +9,19 @@
  *
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation;
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
- *  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
- *  CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
- *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
- *  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
- *  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, 
- *  COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS 
- *  SOFTWARE IS DISCLAIMED.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: sdp_lib.h,v 1.6 2005/01/29 03:08:20 holtmann Exp $
  */
 
 #ifndef __SDP_LIB_H
@@ -107,10 +102,12 @@ int sdp_get_string_attr(const sdp_record_t *rec, uint16_t attr, char *value, int
  * Basic sdp data functions
  */
 sdp_data_t *sdp_data_alloc(uint8_t dtd, const void *value);
+sdp_data_t *sdp_data_alloc_with_length(uint8_t dtd, const void *value, uint32_t length);
 void sdp_data_free(sdp_data_t *data);
 sdp_data_t *sdp_data_get(const sdp_record_t *rec, uint16_t attr_id);
 
 sdp_data_t *sdp_seq_alloc(void **dtds, void **values, int len);
+sdp_data_t *sdp_seq_alloc_with_length(void **dtds, void **values, int *length, int len);
 sdp_data_t *sdp_seq_append(sdp_data_t *seq, sdp_data_t *data);
 
 int sdp_attr_add(sdp_record_t *rec, uint16_t attr, sdp_data_t *data);
@@ -178,6 +175,11 @@ static inline int sdp_set_browse_groups(sdp_record_t *rec, sdp_list_t *seq)
  */
 int sdp_set_access_protos(sdp_record_t *rec, const sdp_list_t *proto);
 
+/*
+ * Set the additional access protocols of the record to those specified in proto
+ */
+int sdp_set_add_access_protos(sdp_record_t *rec, const sdp_list_t *proto);
+
 /*
  * Get protocol port (i.e. PSM for L2CAP, Channel for RFCOMM) 
  */
@@ -286,7 +288,7 @@ void sdp_set_url_attr(sdp_record_t *rec, const char *clientExecURL, const char *
  *    sdp_list_t *rsp_list
  *      This variable is set on a successful return if there are
  *      non-zero service handles. It is a singly linked list of
- *      service records (sdp_record_t)
+ *      service record handles (uint16_t)
  */
 int sdp_service_search_req(sdp_session_t *session, const sdp_list_t *search_list, uint16_t max_rec_num, sdp_list_t **rsp_list);
 
@@ -393,14 +395,17 @@ void sdp_record_free(sdp_record_t *rec);
  * record first and set its attributes using setXXX() methods.
  * 
  * The service provider must then call sdp_record_register() to make 
- * the service record visible to SDP clients.
+ * the service record visible to SDP clients.  This function returns 0
+ * on success or -1 on failure (and sets errno).
  */
 int sdp_record_register(sdp_session_t *sess, sdp_record_t *rec, uint8_t flags);
+int sdp_device_record_register(sdp_session_t *sess, bdaddr_t *device, sdp_record_t *rec, uint8_t flags);
 
 /*
  * Unregister a service record.
  */
 int sdp_record_unregister(sdp_session_t *sess, sdp_record_t *rec);
+int sdp_device_record_unregister(sdp_session_t *sess, bdaddr_t *device, sdp_record_t *rec);
 
 /*
  * Update an existing service record.  (Calling this function
@@ -408,6 +413,7 @@ int sdp_record_unregister(sdp_session_t *sess, sdp_record_t *rec);
  * in an error.)
  */
 int sdp_record_update(sdp_session_t *sess, const sdp_record_t *rec);
+int sdp_device_record_update(sdp_session_t *sess, bdaddr_t *device, const sdp_record_t *rec);
 
 void sdp_record_print(const sdp_record_t *rec);
 
@@ -423,7 +429,7 @@ uuid_t *sdp_uuid_to_uuid128(uuid_t *uuid);
 void sdp_uuid16_to_uuid128(uuid_t *uuid128, uuid_t *uuid16);
 int sdp_uuid128_to_uuid(uuid_t *uuid);
 int sdp_uuid_to_proto(uuid_t *uuid);
-int sdp_uuid_extract(const char *buffer, uuid_t *uuid, int *scanned);
+int sdp_uuid_extract(const uint8_t *buffer, uuid_t *uuid, int *scanned);
 void sdp_uuid_print(const uuid_t *uuid);
 
 #define MAX_LEN_UUID_STR 37
@@ -456,7 +462,7 @@ int sdp_get_access_protos(const sdp_record_t *rec, sdp_list_t **protos);
 /*
  * Get the additional access protocols from the service record
  */
-int sdp_get_add_access_protos(const sdp_record_t *rec, sdp_list_t **pap);
+int sdp_get_add_access_protos(const sdp_record_t *rec, sdp_list_t **protos);
 
 /*
  * Extract the list of browse groups to which the service belongs.
@@ -537,30 +543,30 @@ static inline uint16_t sdp_gen_tid(sdp_session_t *session)
 	return session->tid++;
 }
 
-sdp_record_t *sdp_extract_pdu(const char *pdata, int *scanned);
-sdp_data_t *sdp_extract_string(char *, int *);
+sdp_record_t *sdp_extract_pdu(const uint8_t *pdata, int *scanned);
+sdp_data_t *sdp_extract_string(uint8_t *, int *);
 
 void sdp_data_print(sdp_data_t *data);
 void sdp_print_service_attr(sdp_list_t *alist);
 
 int sdp_attrid_comp_func(const void *key1, const void *key2);
 
-void sdp_set_seq_len(char *ptr, int length);
+void sdp_set_seq_len(uint8_t *ptr, uint32_t length);
 void sdp_set_attrid(sdp_buf_t *pdu, uint16_t id);
 void sdp_append_to_pdu(sdp_buf_t *dst, sdp_data_t *d);
-void sdp_append_to_buf(sdp_buf_t *dst, char *data, int len);
+void sdp_append_to_buf(sdp_buf_t *dst, uint8_t *data, uint32_t len);
 
 int sdp_gen_pdu(sdp_buf_t *pdu, sdp_data_t *data);
 int sdp_gen_record_pdu(const sdp_record_t *rec, sdp_buf_t *pdu);
 
-int sdp_extract_seqtype(const char *buf, uint8_t *dtdp, int *seqlen);
+int sdp_extract_seqtype(const uint8_t *buf, uint8_t *dtdp, int *seqlen);
 
-sdp_data_t *sdp_extract_attr(const char *pdata, int *extractedLength, sdp_record_t *rec);
+sdp_data_t *sdp_extract_attr(const uint8_t *pdata, int *extractedLength, sdp_record_t *rec);
 
 void sdp_pattern_add_uuid(sdp_record_t *rec, uuid_t *uuid);
 void sdp_pattern_add_uuidseq(sdp_record_t *rec, sdp_list_t *seq);
 
-int sdp_send_req_w4_rsp(sdp_session_t *session, char *req, char *rsp, int reqsize, int *rspsize);
+int sdp_send_req_w4_rsp(sdp_session_t *session, uint8_t *req, uint8_t *rsp, uint32_t reqsize, uint32_t *rspsize);
 
 typedef struct {
 	uint8_t length;
