@@ -8,24 +8,19 @@
  *
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation;
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
- *  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
- *  CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
- *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
- *  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
- *  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, 
- *  COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS 
- *  SOFTWARE IS DISCLAIMED.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: hci.c,v 1.80 2005/09/07 16:14:55 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -404,8 +399,8 @@ static hci_map commands_map[] = {
 	{ "Read Number of Supported IAC",		90  },
 	{ "Read Current IAC LAP",			91  },
 	{ "Write Current IAC LAP",			92  },
-	{ "Reserved",					93  },
-	{ "Reserved",					94  },
+	{ "Read Page Scan Period Mode",			93  },
+	{ "Write Page Scan Period Mode",		94  },
 	{ "Read Page Scan Mode",			95  },
 	{ "Write Page Scan Mode",			96  },
 	{ "Set AFH Channel Classification",		97  },
@@ -426,11 +421,11 @@ static hci_map commands_map[] = {
 	{ "Reserved",					112 },
 	{ "Reserved",					113 },
 	{ "Reserved",					114 },
-	{ "Read Loacal Version Information",		115 },
-	{ "Reserved",					116 },
+	{ "Read Local Version Information",		115 },
+	{ "Read Local Supported Commands",		116 },
 	{ "Read Local Supported Features",		117 },
 	{ "Read Local Extended Features",		118 },
-	{ "Read Buffer Sizer",				119 },
+	{ "Read Buffer Size",				119 },
 	{ "Read Country Code",				120 },
 	{ "Read BD ADDR",				121 },
 	{ "Read Failed Contact Counter",		122 },
@@ -449,6 +444,9 @@ static hci_map commands_map[] = {
 	{ "Reserved",					135 },
 	{ "Read Extended Inquiry Response",		136 },
 	{ "Write Extended Inquiry Response",		137 },
+	{ "Unknown",					138 },
+	{ "Unknown",					139 },
+	{ "Sniff Subrate",				140 },
 	{ NULL }
 };
 
@@ -560,6 +558,7 @@ static hci_map lmp_features_map[8][9] = {
 		{ NULL }
 	},
 	{	/* Byte 3 */
+		{ "<no. 24>",		0x01		},	/* Bit 0 */
 		{ "<EDR ACL 2 Mbps>",	LMP_EDR_ACL_2M	},	/* Bit 1 */
 		{ "<EDR ACL 3 Mbps>",	LMP_EDR_ACL_3M	},	/* Bit 2 */
 		{ "<enhanced iscan>",	LMP_ENH_ISCAN	},	/* Bit 3 */
@@ -572,13 +571,18 @@ static hci_map lmp_features_map[8][9] = {
 	{	/* Byte 4 */
 		{ "<EV4 packets>",	LMP_EV4		},	/* Bit 0 */
 		{ "<EV5 packets>",	LMP_EV5		},	/* Bit 1 */
+		{ "<no. 34>",		0x04		},	/* Bit 2 */
 		{ "<AFH cap. slave>",	LMP_AFH_CAP_SLV	},	/* Bit 3 */
 		{ "<AFH class. slave>",	LMP_AFH_CLS_SLV	},	/* Bit 4 */
+		{ "<no. 37>",		0x20		},	/* Bit 5 */
+		{ "<no. 38>",		0x40		},	/* Bit 6 */
 		{ "<3-slot EDR ACL>",	LMP_EDR_3SLOT	},	/* Bit 7 */
 		{ NULL }
 	},
 	{	/* Byte 5 */
 		{ "<5-slot EDR ACL>",	LMP_EDR_5SLOT	},	/* Bit 0 */
+		{ "<sniff subrating>",	LMP_SNIFF_SUBR	},	/* Bit 1 */
+		{ "<no. 42>",		0x04		},	/* Bit 2 */
 		{ "<AFH cap. master>",	LMP_AFH_CAP_MST	},	/* Bit 3 */
 		{ "<AFH class. master>",LMP_AFH_CLS_MST	},	/* Bit 4 */
 		{ "<EDR eSCO 2 Mbps>",	LMP_EDR_ESCO_2M	},	/* Bit 5 */
@@ -647,21 +651,25 @@ int hci_for_each_dev(int flag, int (*func)(int dd, int dev_id, long arg), long a
 	struct hci_dev_list_req *dl;
 	struct hci_dev_req *dr;
 	int dev_id = -1;
-	int i, sk, err;
+	int i, sk, err = 0;
 
 	sk = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI);
 	if (sk < 0)
 		return -1;
 
 	dl = malloc(HCI_MAX_DEV * sizeof(*dr) + sizeof(*dl));
-	if (!dl)
+	if (!dl) {
+		err = errno;
 		goto done;
+	}
 
 	dl->dev_num = HCI_MAX_DEV;
 	dr = dl->dev_req;
 
-	if (ioctl(sk, HCIGETDEVLIST, (void *) dl) < 0)
+	if (ioctl(sk, HCIGETDEVLIST, (void *) dl) < 0) {
+		err = errno;
 		goto free;
+	}
 
 	for (i = 0; i < dl->dev_num; i++, dr++) {
 		if (hci_test_bit(flag, &dr->dev_opt))
@@ -672,13 +680,12 @@ int hci_for_each_dev(int flag, int (*func)(int dd, int dev_id, long arg), long a
 	}
 
 	if (dev_id < 0)
-		errno = ENODEV;
+		err = ENODEV;
 
 free:
 	free(dl);
 
 done:
-	err = errno;
 	close(sk);
 	errno = err;
 
@@ -973,7 +980,13 @@ int hci_send_req(int dd, struct hci_request *r, int to)
 				errno = EIO;
 				goto failed;
 			}
-			break;
+
+			if (r->event != EVT_CMD_STATUS)
+				break;
+
+			r->rlen = MIN(len, r->rlen);
+			memcpy(r->rparam, ptr, r->rlen);
+			goto done;
 
 		case EVT_CMD_COMPLETE:
 			cc = (void *) ptr;
