@@ -9,24 +9,19 @@
  *
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation;
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
- *  IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
- *  CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
- *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
- *  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
- *  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, 
- *  COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS 
- *  SOFTWARE IS DISCLAIMED.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *
- *  $Id: sdp.c,v 1.27 2005/08/07 11:19:58 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -349,10 +344,9 @@ void sdp_uuid_print(const uuid_t *uuid)
 }
 #endif
 
-sdp_data_t *sdp_data_alloc(uint8_t dtd, const void *value)
+sdp_data_t *sdp_data_alloc_with_length(uint8_t dtd, const void *value, uint32_t length)
 {
 	sdp_data_t *seq;
-	int len = 0;
 	sdp_data_t *d = (sdp_data_t *) malloc(sizeof(sdp_data_t));
 
 	if (!d)
@@ -419,21 +413,25 @@ sdp_data_t *sdp_data_alloc(uint8_t dtd, const void *value)
 		d->unitSize += sizeof(uint128_t);
 		break;
 	case SDP_URL_STR8:
-	case SDP_TEXT_STR8:
 	case SDP_URL_STR16:
+	case SDP_TEXT_STR8:
 	case SDP_TEXT_STR16:
-		if (!value)
-			goto out_error;
+		if (!value) {
+			free(d);
+			return NULL;
+		}
 
-		len = strlen(value);
-		d->unitSize += len;
-		if (len <= USHRT_MAX) {
-			d->val.str = (char *) malloc(len + 1);
-			if (!d->val.str)
-				goto out_error;
+		d->unitSize += length;
+		if (length <= USHRT_MAX) {
+			d->val.str = malloc(length);
+			if (!d->val.str) {
+				free(d);
+				return NULL;
+			}
 
-			strcpy(d->val.str, value);
-			if (len <= UCHAR_MAX) {
+			memcpy(d->val.str, value, length);
+
+			if (length <= UCHAR_MAX) {
 				d->unitSize += sizeof(uint8_t);
 				if (dtd != SDP_URL_STR8 && dtd != SDP_TEXT_STR8) {
 					if (dtd == SDP_URL_STR16)
@@ -450,7 +448,8 @@ sdp_data_t *sdp_data_alloc(uint8_t dtd, const void *value)
 			}
 		} else {
 			SDPERR("Strings of size > USHRT_MAX not supported\n");
-			goto out_error;
+			free(d);
+			d = NULL;
 		}
 		break;
 	case SDP_URL_STR32:
@@ -475,13 +474,33 @@ sdp_data_t *sdp_data_alloc(uint8_t dtd, const void *value)
 			d->unitSize += seq->unitSize;
 		break;
 	default:
-		goto out_error;
+		free(d);
+		d = NULL;
 	}
-	return d;
 
-out_error:
-	free(d);
-	return NULL;
+	return d;
+}
+
+sdp_data_t *sdp_data_alloc(uint8_t dtd, const void *value)
+{
+	uint32_t length;
+
+	switch (dtd) {
+	case SDP_URL_STR8:
+	case SDP_URL_STR16:
+	case SDP_TEXT_STR8:
+	case SDP_TEXT_STR16:
+		if (!value)
+			return NULL;
+
+		length = strlen((char *) value);
+		break;
+	default:
+		length = 0;
+		break;
+	}
+
+	return sdp_data_alloc_with_length(dtd, value, length);
 }
 
 sdp_data_t *sdp_seq_append(sdp_data_t *seq, sdp_data_t *d)
@@ -496,6 +515,34 @@ sdp_data_t *sdp_seq_append(sdp_data_t *seq, sdp_data_t *d)
 	return seq;
 }
 
+sdp_data_t *sdp_seq_alloc_with_length(void **dtds, void **values, int *length, int len)
+{
+	sdp_data_t *curr = NULL, *seq = NULL;
+	int i;
+
+	for (i = 0; i < len; i++) {
+		sdp_data_t *data;
+		int8_t dtd = *(uint8_t *) dtds[i];
+
+		if (dtd >= SDP_SEQ8 && dtd <= SDP_ALT32)
+			data = (sdp_data_t *) values[i];
+		else
+			data = sdp_data_alloc_with_length(dtd, values[i], length[i]);
+
+		if (!data)
+			return NULL;
+
+		if (curr)
+			curr->next = data;
+		else
+			seq = data;
+
+		curr = data;
+	}
+
+	return sdp_data_alloc_with_length(SDP_SEQ8, seq, length[i]);
+}
+
 sdp_data_t *sdp_seq_alloc(void **dtds, void **values, int len)
 {
 	sdp_data_t *curr = NULL, *seq = NULL;
@@ -503,19 +550,24 @@ sdp_data_t *sdp_seq_alloc(void **dtds, void **values, int len)
 
 	for (i = 0; i < len; i++) {
 		sdp_data_t *data;
-		uint8_t dtd = *(uint8_t *)dtds[i];
+		uint8_t dtd = *(uint8_t *) dtds[i];
+
 		if (dtd >= SDP_SEQ8 && dtd <= SDP_ALT32)
-			data = (sdp_data_t *)values[i];
+			data = (sdp_data_t *) values[i];
 		else
 			data = sdp_data_alloc(dtd, values[i]);
+
 		if (!data)
 			return NULL;
+
 		if (curr)
 			curr->next = data;
 		else
 			seq = data;
+
 		curr = data;
 	}
+
 	return sdp_data_alloc(SDP_SEQ8, seq);
 }
 
@@ -570,6 +622,7 @@ int sdp_set_data_type(sdp_buf_t *buf, uint8_t dtd)
 
 	*p++ = dtd;
 	buf->data_size += sizeof(uint8_t);
+
 	switch (dtd) {
 	case SDP_SEQ8:
 	case SDP_TEXT_STR8:
@@ -590,6 +643,7 @@ int sdp_set_data_type(sdp_buf_t *buf, uint8_t dtd)
 		buf->data_size += sizeof(uint32_t);
 		break;
 	}
+
 	return buf->data_size - orig;
 }
 
@@ -612,6 +666,7 @@ static int get_data_size(sdp_buf_t *buf, sdp_data_t *sdpdata)
 
 	for (d = sdpdata->val.dataseq; d; d = d->next)
 		n += sdp_gen_pdu(buf, d);
+
 	return n;
 }
 
@@ -627,6 +682,7 @@ int sdp_gen_pdu(sdp_buf_t *buf, sdp_data_t *d)
 	uint8_t *seqp = buf->data + buf->data_size;
 
 	pdu_size = sdp_set_data_type(buf, dtd);
+
 	switch (dtd) {
 	case SDP_DATA_NIL:
 		break;
@@ -680,9 +736,13 @@ int sdp_gen_pdu(sdp_buf_t *buf, sdp_data_t *d)
 		data_size = sizeof(uint128_t);
 		break;
 	case SDP_TEXT_STR8:
-	case SDP_URL_STR8:
 	case SDP_TEXT_STR16:
 	case SDP_TEXT_STR32:
+		src = (unsigned char *)d->val.str;
+		data_size = d->unitSize - sizeof(uint8_t);
+		sdp_set_seq_len(seqp, data_size);
+		break;
+	case SDP_URL_STR8:
 	case SDP_URL_STR16:
 	case SDP_URL_STR32:
 		src = (unsigned char *)d->val.str;
@@ -720,6 +780,7 @@ int sdp_gen_pdu(sdp_buf_t *buf, sdp_data_t *d)
 	default:
 		break;
 	}
+
 	if (!is_seq && !is_alt) {
 		if (src && buf) {
 			memcpy(buf->data + buf->data_size, src, data_size);
@@ -727,7 +788,9 @@ int sdp_gen_pdu(sdp_buf_t *buf, sdp_data_t *d)
 		} else if (dtd != SDP_DATA_NIL)
 			SDPDBG("Gen PDU : Cant copy from NULL source or dest\n");
 	}
+
 	pdu_size += data_size;
+
 	return pdu_size;
 }
 
@@ -1206,9 +1269,10 @@ int sdp_read_rsp(sdp_session_t *session, uint8_t *buf, uint32_t size)
 	fd_set readFds;
 	struct timeval timeout = { SDP_RESPONSE_TIMEOUT, 0 };
 
+	FD_ZERO(&readFds);
 	FD_SET(session->sock, &readFds);
 	SDPDBG("Waiting for response\n");
-	if (0 == select(session->sock + 1, &readFds, NULL, NULL, &timeout)) {
+	if (select(session->sock + 1, &readFds, NULL, NULL, &timeout) == 0) {
 		SDPERR("Client timed out\n");
 		errno = ETIMEDOUT;
 		return -1;
@@ -1529,12 +1593,24 @@ int sdp_get_profile_descs(const sdp_record_t *rec, sdp_list_t **profDescSeq)
 		return -1;
 	}
 	for (seq = sdpdata->val.dataseq; seq && seq->val.dataseq; seq = seq->next) {
-		sdp_data_t *uuid = seq->val.dataseq;
-		sdp_data_t *pVnum = seq->val.dataseq->next;
-		if (uuid && pVnum) {
+		uuid_t *uuid = NULL;
+		uint16_t version = 0x100;
+
+		if (SDP_IS_UUID(seq->dtd)) {
+			uuid = &seq->val.uuid;
+		} else {
+			sdp_data_t *puuid = seq->val.dataseq;
+			sdp_data_t *pVnum = seq->val.dataseq->next;
+			if (puuid && pVnum) {
+				uuid = &puuid->val.uuid;
+				version = pVnum->val.uint16;
+			}
+		}
+
+		if (uuid != NULL) {
 			profDesc = (sdp_profile_desc_t *)malloc(sizeof(sdp_profile_desc_t));
-			profDesc->uuid = uuid->val.uuid;
-			profDesc->version = pVnum->val.uint16;
+			profDesc->uuid = *uuid;
+			profDesc->version = version;
 #ifdef SDP_DEBUG
 			sdp_uuid_print(&profDesc->uuid);
 			SDPDBG("Vnum : 0x%04x\n", profDesc->version);
@@ -1736,13 +1812,31 @@ static sdp_data_t *access_proto_to_dataseq(sdp_record_t *rec, sdp_list_t *proto)
 int sdp_set_access_protos(sdp_record_t *rec, const sdp_list_t *ap)
 {
 	const sdp_list_t *p;
-	sdp_data_t *protos = 0;
+	sdp_data_t *protos = NULL;
 
 	for (p = ap; p; p = p->next) {
-		sdp_data_t *seq = access_proto_to_dataseq(rec, (sdp_list_t *)p->data);
+		sdp_data_t *seq = access_proto_to_dataseq(rec, (sdp_list_t *) p->data);
 		protos = sdp_seq_append(protos, seq);
 	}
+
 	sdp_attr_add(rec, SDP_ATTR_PROTO_DESC_LIST, protos);
+
+	return 0;
+}
+
+int sdp_set_add_access_protos(sdp_record_t *rec, const sdp_list_t *ap)
+{
+	const sdp_list_t *p;
+	sdp_data_t *protos = NULL;
+
+	for (p = ap; p; p = p->next) {
+		sdp_data_t *seq = access_proto_to_dataseq(rec, (sdp_list_t *) p->data);
+		protos = sdp_seq_append(protos, seq);
+	}
+
+	sdp_attr_add(rec, SDP_ATTR_ADD_PROTO_DESC_LIST,
+			protos ? sdp_data_alloc(SDP_SEQ8, protos) : NULL);
+
 	return 0;
 }
 
@@ -3108,6 +3202,10 @@ int sdp_service_search_attr_req(sdp_session_t *session, const sdp_list_t *search
 					status = -1;
 					goto end;
 				}
+				if (!recsize) {
+					sdp_record_free(rec);
+					break;
+				}
 				scanned += recsize;
 				pdata += recsize;
 
@@ -3160,6 +3258,7 @@ static inline int sdp_is_local(const bdaddr_t *device)
 
 sdp_session_t *sdp_connect(const bdaddr_t *src, const bdaddr_t *dst, uint32_t flags)
 {
+	int err;
 	sdp_session_t *session = malloc(sizeof(sdp_session_t));
 	if (!session)
 		return session;
@@ -3204,8 +3303,10 @@ sdp_session_t *sdp_connect(const bdaddr_t *src, const bdaddr_t *dst, uint32_t fl
 		}
 	}
 fail:
+	err = errno;
 	if (session->sock >= 0)
 		close(session->sock);
 	free(session);
+	errno = err;
 	return 0;
 }
